package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.dtos.MixDto
import dev.fyloz.colorrecipesexplorer.dtos.MixMaterialDto
import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.CannotDeleteException
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import javax.persistence.*

@Entity
@Table(name = "mix")
data class Mix(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    var location: String?,

    @ManyToOne
    @JoinColumn(name = "recipe_id")
    val recipe: Recipe,

    @ManyToOne
    @JoinColumn(name = "mix_type_id")
    var mixType: MixType,

    @OneToMany(cascade = [CascadeType.ALL], fetch = FetchType.EAGER, orphanRemoval = true)
    @JoinColumn(name = "mix_id")
    var mixMaterials: Set<MixMaterial>,
) : ModelEntity

// ==== DSL ====
fun mix(
    id: Long? = null,
    location: String? = "location",
    recipe: Recipe = recipe(),
    mixType: MixType = mixType(),
    mixMaterials: MutableSet<MixMaterial> = mutableSetOf(),
    op: Mix.() -> Unit = {}
) = Mix(id, location, recipe, mixType, mixMaterials).apply(op)

@Deprecated("Temporary DSL for transition")
fun mix(
    dto: MixDto
) = Mix(dto.id, dto.location, dto.recipe, mixType(dto.mixType), dto.mixMaterials.map(::mixMaterial).toSet())

@Deprecated("Temporary DSL for transition")
fun mix(
    entity: Mix
) = MixDto(entity.id!!, entity.location, entity.recipe, mixTypeDto(entity.mixType), entity.mixMaterials.map(::mixMaterialDto).toSet())