package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.dtos.MixTypeDto
import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.CannotDeleteException
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import dev.fyloz.colorrecipesexplorer.exception.RestException
import org.springframework.http.HttpStatus
import javax.persistence.*

@Entity
@Table(name = "mix_type")
data class MixType(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    @Column(unique = true)
    val name: String,

    @OneToOne(cascade = [CascadeType.ALL])
    @JoinColumn(name = "material_id")
    var material: Material
) : ModelEntity

// ==== DSL ====
fun mixType(
    id: Long? = null,
    name: String = "name",
    material: Material = material(),
    op: MixType.() -> Unit = {}
) = MixType(id, name, material).apply(op)

fun mixType(
    name: String = "name",
    materialType: MaterialType = materialType(),
    op: MixType.() -> Unit = {}
) = mixType(
    id = null,
    name,
    material = material(name = name, inventoryQuantity = 0f, isMixType = true, materialType = materialType)
).apply(op)

@Deprecated("Temporary DSL for transition")
fun mixTypeDto(
    entity: MixType
) = MixTypeDto(entity.id!!, entity.name, materialDto(entity.material))

@Deprecated("Temporary DSL for transition")
fun mixType(
    dto: MixTypeDto
) = MixType(dto.id, dto.name, material(dto.material))