package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.logic.config.ConfigurationLogic
import dev.fyloz.colorrecipesexplorer.model.ConfigurationBase
import dev.fyloz.colorrecipesexplorer.model.ConfigurationDto
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import dev.fyloz.colorrecipesexplorer.model.account.toAuthority
import dev.fyloz.colorrecipesexplorer.restartApplication
import org.springframework.http.MediaType
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.Authentication
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("api/config")
class ConfigurationController(val configurationLogic: ConfigurationLogic) {
    @GetMapping
    fun getAll(@RequestParam(required = false) keys: String?, authentication: Authentication?) =
        ok(with(configurationLogic) {
            if (keys != null) getAll(keys) else getAll()
        }.filter { authentication.hasAuthority(it) })

    @GetMapping("{key}")
    fun get(@PathVariable key: String, authentication: Authentication?) = with(configurationLogic.get(key)) {
        if (authentication.hasAuthority(this)) ok(this) else forbidden()
    }

    @PutMapping
    @PreAuthorize("hasAuthority('ADMIN')")
    fun set(@RequestBody configurations: List<ConfigurationDto>) = noContent {
        configurationLogic.set(configurations)
    }

    @PostMapping("restart")
    @PreAuthorize("hasAuthority('ADMIN')")
    fun restart() = noContent {
        restartApplication()
    }

    // Icon

    @GetMapping("icon")
    fun getIcon() =
        okFile(configurationLogic.getConfiguredIcon(), MediaType.IMAGE_PNG_VALUE)

    @PutMapping("icon")
    @PreAuthorize("hasAuthority('ADMIN')")
    fun setIcon(@RequestParam icon: MultipartFile) = noContent {
        configurationLogic.setConfiguredIcon(icon)
    }

    // Logo

    @GetMapping("logo")
    fun getLogo() =
        okFile(configurationLogic.getConfiguredLogo(), MediaType.IMAGE_PNG_VALUE)

    @PutMapping("logo")
    @PreAuthorize("hasAuthority('ADMIN')")
    fun setLogo(@RequestParam logo: MultipartFile) = noContent {
        configurationLogic.setConfiguredLogo(logo)
    }
}

private fun Authentication?.hasAuthority(configuration: ConfigurationBase) = when {
    configuration.type.public -> true
    this != null && Permission.ADMIN.toAuthority() in this.authorities -> true
    else -> false
}

