package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.dtos.MaterialTypeDto
import dev.fyloz.colorrecipesexplorer.logic.MaterialTypeLogic
import org.springframework.context.annotation.Profile
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.MATERIAL_TYPE)
@Profile("!emergency")
@PreAuthorizeViewCatalog
class MaterialTypeController(private val materialTypeLogic: MaterialTypeLogic) {
    @GetMapping
    fun getAll() =
        ok(materialTypeLogic.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(materialTypeLogic.getById(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun save(@Valid @RequestBody materialType: MaterialTypeDto) =
        created<MaterialTypeDto>(Constants.ControllerPaths.MATERIAL_TYPE) {
            materialTypeLogic.save(materialType)
        }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun update(@Valid @RequestBody materialType: MaterialTypeDto) =
        noContent {
            materialTypeLogic.updateNonSystemType(materialType)
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            materialTypeLogic.deleteById(id)
        }
}

