package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeEditRecipes
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewRecipes
import dev.fyloz.colorrecipesexplorer.dtos.MixDto
import dev.fyloz.colorrecipesexplorer.dtos.MixSaveDto
import dev.fyloz.colorrecipesexplorer.logic.MixLogic
import org.springframework.context.annotation.Profile
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.MIX)
@Profile("!emergency")
@PreAuthorizeViewRecipes
class MixController(private val mixLogic: MixLogic) {
    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(mixLogic.getById(id))

    @PostMapping
    @PreAuthorizeEditRecipes
    fun save(@Valid @RequestBody mix: MixSaveDto) =
        created<MixDto>(Constants.ControllerPaths.MIX) {
            mixLogic.save(mix)
        }

    @PutMapping
    @PreAuthorizeEditRecipes
    fun update(@Valid @RequestBody mix: MixSaveDto) =
        noContent {
            mixLogic.update(mix)
        }

    @DeleteMapping("{id}")
    @PreAuthorizeEditRecipes
    fun deleteById(@PathVariable id: Long) =
        noContent {
            mixLogic.deleteById(id)
        }
}