package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionError
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionsException

object PositionUtils {
    const val INVALID_FIRST_POSITION_ERROR_CODE = "first"
    const val DUPLICATED_POSITION_ERROR_CODE = "duplicated"
    const val GAP_BETWEEN_POSITIONS_ERROR_CODE = "gap"

    private const val FIRST_POSITION = 1

    fun validate(positions: List<Int>) {
        if (positions.isEmpty()) {
            return
        }

        val sortedPositions = positions.sorted()
        val errors = mutableSetOf<InvalidPositionError>()

        validateFirstPosition(sortedPositions[0], errors)
        validateDuplicatedPositions(sortedPositions, errors)
        validateGapsInPositions(sortedPositions, errors)

        if (errors.isNotEmpty()) {
            throw InvalidPositionsException(errors)
        }
    }

    private fun validateFirstPosition(position: Int, errors: MutableSet<InvalidPositionError>) {
        if (position == FIRST_POSITION) {
            return
        }

        errors += InvalidPositionError(INVALID_FIRST_POSITION_ERROR_CODE, "The first position must be $FIRST_POSITION")
    }

    private fun validateDuplicatedPositions(positions: List<Int>, errors: MutableSet<InvalidPositionError>) {
        errors += positions.findDuplicated()
            .map { InvalidPositionError(DUPLICATED_POSITION_ERROR_CODE, "The position $it is duplicated") }
    }

    private fun validateGapsInPositions(positions: List<Int>, errors: MutableSet<InvalidPositionError>) {
        if (!positions.hasGaps()) {
            return
        }

        errors += InvalidPositionError(GAP_BETWEEN_POSITIONS_ERROR_CODE, "There is a gap between the positions")
    }
}