package dev.fyloz.colorrecipesexplorer.logic

import dev.fyloz.colorrecipesexplorer.dtos.CompanyDto
import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.CannotDeleteException
import dev.fyloz.colorrecipesexplorer.service.CompanyService
import io.mockk.clearAllMocks
import io.mockk.every
import io.mockk.mockk
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows

class DefaultCompanyLogicTest {
    private val companyServiceMock = mockk<CompanyService>()

    private val companyLogic = DefaultCompanyLogic(companyServiceMock)

    private val company = CompanyDto(id = 1L, name = "UnitTestCompany")

    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun save_nameExists_throwsAlreadyExistsException() {
        // Arrange
        every { companyServiceMock.existsByName(any(), any()) } returns true

        // Act
        // Assert
        assertThrows<AlreadyExistsException> { companyLogic.save(company) }
    }

    @Test
    fun update_nameExists_throwsAlreadyExistsException() {
        // Arrange
        every { companyServiceMock.existsByName(any(), any()) } returns true

        // Act
        // Assert
        assertThrows<AlreadyExistsException> { companyLogic.update(company) }
    }

    @Test
    fun deleteById_recipesDependsOnCompany_throwsCannotDeleteException() {
        // Arrange
        every { companyServiceMock.isUsedByRecipe(company.id) } returns true

        // Act
        // Assert
        assertThrows<CannotDeleteException> { companyLogic.deleteById(company.id) }
    }
}